@@ -70,7 +70,7 @@ struct osmo_pcap_client_conn {
 	char *srv_ip;
 	int srv_port;
 	char *source_ip;
-	struct osmo_wqueue wqueue;
+	struct osmo_stream_cli *cli;
 	struct osmo_timer_list timer;
 	enum osmo_pcap_protocol protocol;
 
@@ -87,6 +87,7 @@ struct osmo_pcap_client_conn {
 	unsigned tls_log_level;
 
 	struct osmo_tls_session tls_session;
+	struct osmo_wqueue wqueue;
 
 	/* back pointer */
 	struct osmo_pcap_client *client;
@@ -148,6 +149,7 @@ void osmo_client_conn_send_link(struct osmo_pcap_client_conn *conn);
 void osmo_client_conn_connect(struct osmo_pcap_client_conn *conn);
 void osmo_client_conn_disconnect(struct osmo_pcap_client_conn *conn);
 void osmo_client_conn_reconnect(struct osmo_pcap_client_conn *conn);
+bool osmo_client_conn_is_connected(const struct osmo_pcap_client_conn *conn);
 
 
 struct osmo_pcap_handle *osmo_pcap_handle_alloc(struct osmo_pcap_client *client, const char *devname);
