@@ -19,11 +19,9 @@
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  *
  */
+#pragma once
 
-#ifndef WIREFORMAT_H
-#define WIREFORMAT_H
-
-#include <inttypes.h>
+#include <stdint.h>
 #include <pcap.h>
 
 enum OsmoPcapDataType {
@@ -49,5 +47,3 @@ struct osmo_pcap_pkthdr {
 	uint32_t caplen;
 	uint32_t len;
 } __attribute__((packed));
-
-#endif
