@@ -52,29 +52,6 @@ static int daemonize = 0;
 void *tall_cli_ctx;
 struct osmo_pcap_client *pcap_client;
 
-
-static const struct rate_ctr_desc pcap_client_ctr_desc[] = {
-	[CLIENT_CTR_CONNECT]		= { "server:connect",		"Connects to the server" },
-	[CLIENT_CTR_BYTES]		= { "captured:bytes",		"Captured bytes        " },
-	[CLIENT_CTR_PKTS]		= { "captured:pkts",		"Captured packets      " },
-	[CLIENT_CTR_2BIG]		= { "bpf:too_big",		"Captured data too big " },
-	[CLIENT_CTR_NOMEM]		= { "client:no_mem",		"No memory available   " },
-	[CLIENT_CTR_QERR]		= { "client:queue_err",		"Can not queue data    " },
-	[CLIENT_CTR_PERR]		= { "client:pcap_err",		"libpcap error         " },
-	[CLIENT_CTR_WERR]		= { "client:write_err",		"Write error           " },
-	[CLIENT_CTR_P_RECV]		= { "pcap:recv",		"PCAP received packets " },
-	[CLIENT_CTR_P_DROP]		= { "pcap:drop",		"PCAP dropped packets  " },
-	[CLIENT_CTR_P_IFDROP]		= { "pcap:ifdrop",		"iface dropped packets " },
-};
-
-static const struct rate_ctr_group_desc pcap_client_ctr_group_desc = {
-	.group_name_prefix		= "pcap:client",
-	.group_description		= "PCAP Client statistics",
-	.num_ctr			= ARRAY_SIZE(pcap_client_ctr_desc),
-	.ctr_desc			= pcap_client_ctr_desc,
-	.class_id			= OSMO_STATS_CLASS_GLOBAL,
-};
-
 static struct vty_app_info vty_info = {
 	.name		= "OsmoPCAPClient",
 	.version	= PACKAGE_VERSION,
@@ -261,17 +238,8 @@ int main(int argc, char **argv)
 		exit(1);
 	}
 
-	/* initialize the queue */
-	INIT_LLIST_HEAD(&pcap_client->conns);
-	osmo_client_conn_init(&pcap_client->conn, pcap_client);
-	pcap_client->conn.name = "default";
-
-	/* initialize the stats interface */
-	pcap_client->ctrg = rate_ctr_group_alloc(pcap_client, &pcap_client_ctr_group_desc, 0);
-	if (!pcap_client->ctrg) {
-		LOGP(DCLIENT, LOGL_ERROR, "Failed to allocate rate ctr\n");
-		exit(1);
-	}
+	/* Default conn, always present. */
+	pcap_client->conn = osmo_client_conn_alloc(pcap_client, "default");
 
 	if (vty_read_config_file(config_file, NULL) < 0) {
 		LOGP(DCLIENT, LOGL_ERROR,
@@ -285,10 +253,15 @@ int main(int argc, char **argv)
 		exit(1);
 	}
 
+	rc = osmo_client_start_capture(pcap_client);
+	if (rc < 0) {
+		LOGP(DCLIENT, LOGL_ERROR, "Failed to start capturing on interfaces\n");
+		exit(1);
+	}
 
 	/* attempt to connect to the remote */
-	if (pcap_client->conn.srv_ip && pcap_client->conn.srv_port > 0)
-		osmo_client_connect(&pcap_client->conn);
+	if (pcap_client->conn->srv_ip && pcap_client->conn->srv_port > 0)
+		osmo_client_conn_connect(pcap_client->conn);
 
 	if (daemonize) {
 		rc = osmo_daemonize();
